
RegisterInterfaceCommand = "register_interface_command"
RegisterInterfaceResponse = "register_interface_response"
RegisterFunctionCommand = "register_function_command"
RegisterFunctionResponse = "register_function_response"
CallFunctionCommand = "call_function_command"
CallFunctionResponse = "call_function_response"
RunFunctionCommand = "run_function_command"
RunFunctionResponse = "run_function_response"
ErrorResponse = "error_response"
RegisterEventCommand = "register_event_command"
RegisterEventResponse = "register_event_response"
RegisterListenerCommand = "register_listener_command"
RegisterListenerResponse = "register_listener_response"
DeregisterListenerCommand = "deregister_listener_command"
DeregisterListenerResponse = "deregister_listener_response"
FireEventCommand = "fire_event_command"
RegisterObjectCommand = "register_object_command"
RegisterObjectResponse = "register_object_response"
WatchObjectCommand = "watch_object_command"
WatchObjectResponse = "watch_object_response"
UnwatchObjectCommand = "unwatch_object_command"
UnwatchObjectResponse = "unwatch_object_response"
SetObjectCommand = "set_object_command"
GetObjectValueCommand = "get_object_value_command"
GetObjectValueResponse = "get_object_value_response"
PingCommand = "ping_command"
PingResponse = "ping_response"
ActivateHackCommand = "activate_hack_command"
ActivateHackResponse = "activate_hack_response"
