
# FIXME: change everything that points to the proxy module to point to the
# single and multiple modules instead, then get rid of these two wildcard
# import lines

from autobus2.proxy.single import *
from autobus2.proxy.multiple import *

