
"""
This is a pseudo-module to hold all widget classes generated by librtk for
any given connection. The classes aren't really present in this module; it
simply exists as a placeholder for the module that these generated classes
appear to be defined in.
"""
