#!/usr/bin/env python

# DO NOT EDIT THIS FILE. Edit setup.py.template instead, then
# run update-setup-script to update this file.

#!/usr/bin/env python

from distutils.core import setup

version = "0.1.2012.05.17.21.57" # !!!VERSION!!!

setup(
    name="afn",
    version=version,
    author="Alexander Boyd",
    author_email="alex@opengroove.org",
    description="A collection of useful tools and libraries",
    long_description=
"""
**AFN** is Alexander Boyd's "labs" project. It contains a collection of useful
tools and libraries that aren't big enough or popular enough to merit their own
projects. 
""",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    packages=[
        # !!!PACKAGES!!!
        'traytimer',
        'uofu',
        'autobus2.landfill',
        'autobus2.proxy',
        'autobus2.examples',
        'autobus2',
        'autobus2.tools',
        'libautobus',
        'librtk.examples',
        'librtk.schema',
        'librtk',
        'afn.ucreditu',
        'afn.landfill',
        'afn.dnsd.examples',
        'afn.dnsd',
        'afn.backports',
        'afn',
        'afn.utils',
        'afn.utils.cad',
        'afn.tools',
        'afn.bf',
        'afntest',
        'activehomed',
        'autobus',
    ],
    package_dir={"": "src"}
)
